@@ -2,8 +2,10 @@ package OpenGuides::RDF;
 
 use strict;
 
+use OpenGuides::Utils;
+
 use vars qw( $VERSION );
-$VERSION = '0.09';
+$VERSION = '0.12';
 
 use Time::Piece;
 use URI::Escape;
@@ -80,13 +82,12 @@ sub emit_rdfxml {
                    );
 
     foreach my $var ( qw( phone fax website opening_hours_text address
-                          postcode city country latitude longitude username
-                          os_x os_y summary ) ) {
+                          postcode city country latitude longitude
+                          os_x os_y map_link summary node_image ) ) {
         my $val = $metadata{$var}[0] || $defaults{$var} || "";
         $tt_vars{$var} = $val;
     }
 
-    
     my @cats = @{ $metadata{category} || [] };
     @cats = map { { name => $_ } } @cats;
     $tt_vars{categories} = \@cats;
@@ -103,7 +104,7 @@ sub emit_rdfxml {
     # Check for geospatialness and define container object as appropriate.
     my $is_geospatial;
     foreach my $var ( qw( os_x os_y latitude longitude address postcode
-                          opening_hours_text ) ) {
+                          opening_hours_text map_link ) ) {
         $is_geospatial = 1 if $tt_vars{$var};
     }
 
@@ -113,6 +114,15 @@ sub emit_rdfxml {
                                         : "rdf:Description";
     $tt_vars{is_geospatial} = $is_geospatial;
 
+    # Fix up lat and long.
+    eval {
+           @tt_vars{ qw( wgs84_long wgs84_lat ) } =
+               OpenGuides::Utils->get_wgs84_coords(
+                                             longitude => $tt_vars{longitude},
+                                             latitude  => $tt_vars{latitude},
+                                             config    => $config );
+    };
+
     # Timestamp of last edited.
     my $timestamp = $node_data{last_modified};
     if ( $timestamp ) {
@@ -122,17 +132,14 @@ sub emit_rdfxml {
         $tt_vars{timestamp} = $time->strftime("%Y-%m-%dT%H:%M:%S");
     }
 
-    $tt_vars{user_id} = $tt_vars{username};
-    $tt_vars{user_id} =~ s/\s/_/g;
-    
     $tt_vars{node_uri} = $self->{make_node_url}->( $node_name );
     $tt_vars{node_uri_with_version}
                             = $self->{make_node_url}->( $node_name,
                                                         $tt_vars{version} );
 
-    # Should probably be moved into OpenGuides::Utils.
-    if ($node_data{content} =~ /^\#REDIRECT \[\[(.*?)]\]$/) {
-        my $redirect = $1;
+    my $redirect = OpenGuides::Utils->detect_redirect( content =>
+                                                         $node_data{content} );
+    if ( $redirect ) {
         $tt_vars{redirect} = $config->script_url . $config->script_name
                              . "?id="
                              . $formatter->node_name_to_node_param( $redirect )
@@ -146,12 +153,36 @@ sub emit_rdfxml {
         }
     }
 
+    my @revisions = $wiki->list_node_all_versions(
+                                                   name => $node_name,
+                                                   with_content => 0,
+                                                   with_metadata => 1,
+                                                 );
+
+    # We want all users who have edited the page listed as contributors,
+    # but only once each
+    foreach my $rev ( @revisions ) {
+        my $username = $rev->{metadata}{username};
+        next unless defined $username && length $username;
+
+        my $user_id = $username;
+        $user_id =~ s/\s+/_/g;
+        
+        $tt_vars{contributors}{$username} ||=
+            {
+              username => encode_entities_numeric($username),
+              user_id  => encode_entities_numeric($user_id),
+            };
+    }
+
     # OK, we've set all our template variables; now process the template.
     my $template_path = $config->template_path;
     my $custom_template_path = $config->custom_template_path || "";
     my $tt = Template->new( {
                     INCLUDE_PATH => "$custom_template_path:$template_path" } );
 
+    $tt_vars{full_cgi_url} = $config->script_url . $config->script_name;
+
     my $rdf;
     $tt->process( "node_rdf.tt", \%tt_vars, \$rdf );
     $rdf ||= "ERROR: " . $tt->error;
@@ -253,7 +284,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2007 The OpenGuides Project.  All Rights Reserved.
+Copyright (C) 2003-2008 The OpenGuides Project.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
